@@ -24,6 +24,7 @@
  */
 #include <algorithm>
 #include <cstdlib>
+#include <climits>
 
 /* ap_config.h checks whether the compiler has support for C99's designated
  * initializers, and defines AP_HAVE_DESIGNATED_INITIALIZER if it does. However,
@@ -185,20 +186,12 @@ create_dir_config_struct(apr_pool_t *pool) {
 void *
 passenger_config_create_dir(apr_pool_t *p, char *dirspec) {
 	DirConfig *config = create_dir_config_struct(p);
-	config->enabled = DirConfig::UNSET;
-	config->ruby = NULL;
-	config->python = NULL;
-	config->environment = NULL;
+	
+	#include "CreateDirConfig.cpp"
+
 	config->appRoot = NULL;
-	config->user = NULL;
-	config->group = NULL;
 	config->spawnMethod = DirConfig::SM_UNSET;
 	config->maxPreloaderIdleTime = -1;
-	config->maxRequests = 0;
-	config->maxRequestsSpecified = false;
-	config->minInstances = 1;
-	config->minInstancesSpecified = false;
-	config->highPerformance = DirConfig::UNSET;
 	config->resolveSymlinksInDocRoot = DirConfig::UNSET;
 	config->allowEncodedSlashes = DirConfig::UNSET;
 	config->statThrottleRate = 0;
@@ -218,25 +211,17 @@ passenger_config_merge_dir(apr_pool_t *p, void *basev, void *addv) {
 	DirConfig *base = (DirConfig *) basev;
 	DirConfig *add = (DirConfig *) addv;
 	
-	config->enabled = (add->enabled == DirConfig::UNSET) ? base->enabled : add->enabled;
+	#include "MergeDirConfig.cpp"
 	
 	config->baseURIs = base->baseURIs;
 	for (set<string>::const_iterator it(add->baseURIs.begin()); it != add->baseURIs.end(); it++) {
 		config->baseURIs.insert(*it);
 	}
 	
-	MERGE_STR_CONFIG(ruby);
-	MERGE_STR_CONFIG(python);
-	MERGE_STR_CONFIG(environment);
 	MERGE_STR_CONFIG(appRoot);
 	MERGE_STRING_CONFIG(appGroupName);
-	MERGE_STR_CONFIG(user);
-	MERGE_STR_CONFIG(group);
 	config->spawnMethod = (add->spawnMethod == DirConfig::SM_UNSET) ? base->spawnMethod : add->spawnMethod;
 	config->maxPreloaderIdleTime = (add->maxPreloaderIdleTime == -1) ? base->maxPreloaderIdleTime : add->maxPreloaderIdleTime;
-	MERGE_INT_CONFIG(maxRequests);
-	MERGE_INT_CONFIG(minInstances);
-	MERGE_THREEWAY_CONFIG(highPerformance);
 	MERGE_INT_CONFIG(statThrottleRate);
 	MERGE_STR_CONFIG(restartDir);
 	MERGE_STR_CONFIG(uploadBufferDir);
@@ -266,7 +251,6 @@ DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_default_ruby, defaultRuby)
 DEFINE_SERVER_INT_CONFIG_SETTER(cmd_passenger_log_level, logLevel, unsigned int, 0)
 DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_debug_log_file, debugLogFile)
 DEFINE_SERVER_INT_CONFIG_SETTER(cmd_passenger_max_pool_size, maxPoolSize, unsigned int, 1)
-DEFINE_SERVER_INT_CONFIG_SETTER(cmd_passenger_max_instances_per_app, maxInstancesPerApp, unsigned int, 0)
 DEFINE_SERVER_INT_CONFIG_SETTER(cmd_passenger_pool_idle_time, poolIdleTime, unsigned int, 0)
 DEFINE_SERVER_BOOLEAN_CONFIG_SETTER(cmd_passenger_user_switching, userSwitching)
 DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_default_user, defaultUser)
@@ -291,18 +275,11 @@ cmd_passenger_pre_start(cmd_parms *cmd, void *pcfg, const char *arg) {
 	return NULL;
 }
 
-DEFINE_DIR_INT_CONFIG_SETTER(cmd_passenger_min_instances, minInstances, unsigned long, 0)
-DEFINE_DIR_INT_CONFIG_SETTER(cmd_passenger_max_requests, maxRequests, unsigned long, 0)
-DEFINE_DIR_THREEWAY_CONFIG_SETTER(cmd_passenger_high_performance, highPerformance)
-DEFINE_DIR_THREEWAY_CONFIG_SETTER(cmd_passenger_enabled, enabled)
-DEFINE_DIR_STR_CONFIG_SETTER(cmd_passenger_ruby, ruby)
-DEFINE_DIR_STR_CONFIG_SETTER(cmd_passenger_python, python)
-DEFINE_DIR_STR_CONFIG_SETTER(cmd_environment, environment)
+#include "ConfigurationSetters.cpp"
+
 DEFINE_DIR_INT_CONFIG_SETTER(cmd_passenger_stat_throttle_rate, statThrottleRate, unsigned long, 0)
 DEFINE_DIR_STR_CONFIG_SETTER(cmd_passenger_app_root, appRoot)
 DEFINE_DIR_STR_CONFIG_SETTER(cmd_passenger_app_group_name, appGroupName)
-DEFINE_DIR_STR_CONFIG_SETTER(cmd_passenger_user, user)
-DEFINE_DIR_STR_CONFIG_SETTER(cmd_passenger_group, group)
 DEFINE_DIR_STR_CONFIG_SETTER(cmd_passenger_restart_dir, restartDir)
 DEFINE_DIR_STR_CONFIG_SETTER(cmd_passenger_upload_buffer_dir, uploadBufferDir)
 DEFINE_DIR_STR_CONFIG_SETTER(cmd_union_station_key, unionStationKey)
@@ -451,11 +428,6 @@ const command_rec passenger_commands[] = {
 		NULL,
 		RSRC_CONF,
 		"The default Ruby interpreter to use."),
-	AP_INIT_TAKE1("PassengerRuby",
-		(Take1Func) cmd_passenger_ruby,
-		NULL,
-		OR_OPTIONS | ACCESS_CONF,
-		"The Ruby interpreter to use."),
 	AP_INIT_TAKE1("PassengerLogLevel",
 		(Take1Func) cmd_passenger_log_level,
 		NULL,
@@ -471,16 +443,6 @@ const command_rec passenger_commands[] = {
 		NULL,
 		RSRC_CONF,
 		"The maximum number of simultaneously alive application instances."),
-	AP_INIT_TAKE1("PassengerMinInstances",
-		(Take1Func) cmd_passenger_min_instances,
-		NULL,
-		OR_LIMIT | ACCESS_CONF | RSRC_CONF,
-		"The minimum number of application instances to keep when cleaning idle instances."),
-	AP_INIT_TAKE1("PassengerMaxInstancesPerApp",
-		(Take1Func) cmd_passenger_max_instances_per_app,
-		NULL,
-		RSRC_CONF,
-		"The maximum number of simultaneously alive application instances a single application may occupy."),
 	AP_INIT_TAKE1("PassengerPoolIdleTime",
 		(Take1Func) cmd_passenger_pool_idle_time,
 		NULL,
@@ -491,16 +453,6 @@ const command_rec passenger_commands[] = {
 		NULL,
 		RSRC_CONF,
 		"Whether to enable user switching support."),
-	AP_INIT_TAKE1("PassengerUser",
-		(Take1Func) cmd_passenger_user,
-		NULL,
-		ACCESS_CONF | RSRC_CONF,
-		"The user that Ruby applications must run as."),
-	AP_INIT_TAKE1("PassengerGroup",
-		(Take1Func) cmd_passenger_group,
-		NULL,
-		ACCESS_CONF | RSRC_CONF,
-		"The group that Ruby applications must run as."),
 	AP_INIT_TAKE1("PassengerDefaultUser",
 		(Take1Func) cmd_passenger_default_user,
 		NULL,
@@ -516,21 +468,54 @@ const command_rec passenger_commands[] = {
 		NULL,
 		RSRC_CONF,
 		"The temp directory that Passenger should use."),
-	AP_INIT_TAKE1("PassengerMaxRequests",
-		(Take1Func) cmd_passenger_max_requests,
+	AP_INIT_TAKE1("PassengerMaxPreloaderIdleTime",
+		(Take1Func) cmd_passenger_max_preloader_idle_time,
 		NULL,
-		OR_LIMIT | ACCESS_CONF | RSRC_CONF,
-		"The maximum number of requests that an application instance may process."),
-	AP_INIT_FLAG("PassengerHighPerformance",
-		(FlagFunc) cmd_passenger_high_performance,
+		RSRC_CONF,
+		"The maximum number of seconds that a preloader process may be idle before it is shutdown."),
+	AP_INIT_TAKE1("UnionStationGatewayAddress",
+		(Take1Func) cmd_union_station_gateway_address,
 		NULL,
-		OR_ALL,
-		"Enable or disable Passenger's high performance mode."),
-	AP_INIT_FLAG("PassengerEnabled",
-		(FlagFunc) cmd_passenger_enabled,
+		RSRC_CONF,
+		"The Union Station Gateway host name."),
+	AP_INIT_TAKE1("UnionStationGatewayPort",
+		(Take1Func) cmd_union_station_gateway_port,
 		NULL,
-		OR_ALL,
-		"Enable or disable Phusion Passenger."),
+		RSRC_CONF,
+		"The Union Station Gateway port number."),
+	AP_INIT_TAKE1("UnionStationGatewayCert",
+		(Take1Func) cmd_union_station_gateway_cert,
+		NULL,
+		RSRC_CONF,
+		"The Union Station Gateway certificate."),
+	AP_INIT_TAKE1("UnionStationProxyAddress",
+		(Take1Func) cmd_union_station_proxy_address,
+		NULL,
+		RSRC_CONF,
+		"The address of the proxy that should be used for sending data to Union Station."),
+	AP_INIT_TAKE1("PassengerAnalyticsLogUser",
+		(Take1Func) cmd_passenger_analytics_log_user,
+		NULL,
+		RSRC_CONF,
+		"The owner of analytics files."),
+	AP_INIT_TAKE1("PassengerAnalyticsLogGroup",
+		(Take1Func) cmd_passenger_analytics_log_group,
+		NULL,
+		RSRC_CONF,
+		"The group of analytics files."),
+	AP_INIT_TAKE1("PassengerPreStart",
+		(Take1Func) cmd_passenger_pre_start,
+		NULL,
+		RSRC_CONF,
+		"Prestart the given web applications during startup."),
+	AP_INIT_TAKE1("PassengerSpawnMethod",
+		(Take1Func) cmd_passenger_spawn_method,
+		NULL,
+		RSRC_CONF,
+		"The spawn method to use."),
+	
+	#include "ConfigurationCommands.cpp"
+	
 	AP_INIT_TAKE1("PassengerAppGroupName",
 		(Take1Func) cmd_passenger_app_group_name,
 		NULL,
@@ -586,62 +571,17 @@ const command_rec passenger_commands[] = {
 		NULL,
 		OR_OPTIONS | ACCESS_CONF | RSRC_CONF,
 		"Whether to display friendly error pages when something goes wrong"),
-	AP_INIT_TAKE1("PassengerSpawnMethod",
-		(Take1Func) cmd_passenger_spawn_method,
-		NULL,
-		RSRC_CONF,
-		"The spawn method to use."),
 	AP_INIT_TAKE1("PassengerBaseURI",
 		(Take1Func) cmd_passenger_base_uri,
 		NULL,
 		OR_OPTIONS | ACCESS_CONF | RSRC_CONF,
 		"Declare the given base URI as belonging to an application."),
-	AP_INIT_TAKE1("PassengerMaxPreloaderIdleTime",
-		(Take1Func) cmd_passenger_max_preloader_idle_time,
-		NULL,
-		RSRC_CONF,
-		"The maximum number of seconds that a preloader process may be idle before it is shutdown."),
-	AP_INIT_TAKE1("UnionStationGatewayAddress",
-		(Take1Func) cmd_union_station_gateway_address,
-		NULL,
-		RSRC_CONF,
-		"The Union Station Gateway host name."),
-	AP_INIT_TAKE1("UnionStationGatewayPort",
-		(Take1Func) cmd_union_station_gateway_port,
-		NULL,
-		RSRC_CONF,
-		"The Union Station Gateway port number."),
-	AP_INIT_TAKE1("UnionStationGatewayCert",
-		(Take1Func) cmd_union_station_gateway_cert,
-		NULL,
-		RSRC_CONF,
-		"The Union Station Gateway certificate."),
-	AP_INIT_TAKE1("UnionStationProxyAddress",
-		(Take1Func) cmd_union_station_proxy_address,
-		NULL,
-		RSRC_CONF,
-		"The address of the proxy that should be used for sending data to Union Station."),
-	AP_INIT_TAKE1("PassengerAnalyticsLogUser",
-		(Take1Func) cmd_passenger_analytics_log_user,
-		NULL,
-		RSRC_CONF,
-		"The owner of analytics files."),
-	AP_INIT_TAKE1("PassengerAnalyticsLogGroup",
-		(Take1Func) cmd_passenger_analytics_log_group,
-		NULL,
-		RSRC_CONF,
-		"The group of analytics files."),
-	AP_INIT_TAKE1("PassengerPreStart",
-		(Take1Func) cmd_passenger_pre_start,
-		NULL,
-		RSRC_CONF,
-		"Prestart the given web applications during startup."),
 	AP_INIT_FLAG("UnionStationSupport",
 		(Take1Func) cmd_union_station_support,
 		NULL,
 		OR_OPTIONS | ACCESS_CONF | RSRC_CONF,
 		"Whether to enable logging through Union Station."),
-	
+
 	/*****************************/
 	AP_INIT_TAKE1("PassengerMemoryLimit",
 		(Take1Func) cmd_passenger_enterprise_only,
@@ -684,27 +624,6 @@ const command_rec passenger_commands[] = {
 		OR_ALL,
 		"The number of threads that Phusion Passenger should spawn per application."),
 
-	// Rails-specific settings.
-	AP_INIT_TAKE1("RailsEnv",
-		(Take1Func) cmd_environment,
-		NULL,
-		OR_OPTIONS | ACCESS_CONF | RSRC_CONF,
-		"The environment under which a Rails app must run."),
-	
-	// Rack-specific settings.
-	AP_INIT_TAKE1("RackEnv",
-		(Take1Func) cmd_environment,
-		NULL,
-		OR_OPTIONS | ACCESS_CONF | RSRC_CONF,
-		"The environment under which a Rack app must run."),
-	
-	// WSGI-specific settings.
-	AP_INIT_TAKE1("PassengerPython",
-		(Take1Func) cmd_passenger_python,
-		NULL,
-		OR_OPTIONS | ACCESS_CONF | RSRC_CONF,
-		"The Python interpreter to use."),
-	
 	// Backwards compatibility options.
 	AP_INIT_TAKE1("RailsRuby",
 		(Take1Func) cmd_passenger_ruby,
