@@ -45,7 +45,10 @@
 #include <cstdio>
 #include <unistd.h>
 
+#include <oxt/initialize.hpp>
 #include <oxt/macros.hpp>
+#include <oxt/backtrace.hpp>
+#include <oxt/detail/context.hpp>
 #include "Hooks.h"
 #include "Bucket.h"
 #include "Configuration.hpp"
@@ -480,6 +483,17 @@ private:
 	int handleRequest(request_rec *r) {
 		/********** Step 1: preparation work **********/
 		
+		/* Initialize OXT backtrace support if not already done for this thread */
+		if (oxt::get_thread_local_context() == NULL) {
+			/* There is no need to cleanup the context. Apache uses a static
+			 * number of threads per process.
+			 */
+			thread_local_context_ptr context = thread_local_context::make_shared_ptr();
+			unsigned long tid = (unsigned long) pthread_self();
+			context->thread_name = "Worker " + integerToHex(tid);
+			oxt::set_thread_local_context(context);
+		}
+
 		/* Check whether an error occured in prepareRequest() that should be reported
 		 * to the browser.
 		 */
@@ -520,19 +534,21 @@ private:
 			/********** Step 2: handle HTTP upload data, if any **********/
 			
 			int httpStatus = ap_setup_client_block(r, REQUEST_CHUNKED_DECHUNK);
-	    		if (httpStatus != OK) {
+	    	if (httpStatus != OK) {
 				return httpStatus;
 			}
 			
 			this_thread::disable_interruption di;
 			this_thread::disable_syscall_interruption dsi;
 			bool expectingUploadData;
+			bool shouldBufferUploads;
 			string uploadDataMemory;
-			shared_ptr<BufferedUpload> uploadDataFile;
+			boost::shared_ptr<BufferedUpload> uploadDataFile;
 			const char *contentLength;
 			
 			expectingUploadData = ap_should_client_block(r);
 			contentLength = lookupHeader(r, "Content-Length");
+			shouldBufferUploads = config->bufferUpload != DirConfig::DISABLED;
 			
 			/* If the HTTP upload data is larger than a threshold, or if the HTTP
 			 * client sent HTTP upload data using the "chunked" transfer encoding
@@ -543,17 +559,15 @@ private:
 			 * the HTTP client might block indefinitely until it's done uploading.
 			 * This would quickly exhaust the application pool.
 			 */
-			if (expectingUploadData) {
+			if (expectingUploadData && shouldBufferUploads) {
 				if (contentLength == NULL || atol(contentLength) > LARGE_UPLOAD_THRESHOLD) {
 					uploadDataFile = receiveRequestBody(r);
 				} else {
 					receiveRequestBody(r, contentLength, uploadDataMemory);
 				}
-			}
-			
-			if (expectingUploadData) {
+
 				/* We'll set the Content-Length header to the length of the
-				 * received upload data. Rails requires this header for its
+				 * received upload data. Rails 2 requires this header for its
 				 * HTTP upload data multipart parsing process.
 				 * There are two reasons why we don't rely on the Content-Length
 				 * header as sent by the client:
@@ -592,16 +606,20 @@ private:
 			sizeString[ret] = '\0';
 			requestData[0] = StaticString(sizeString, ret);
 			
-			if (expectingUploadData && uploadDataFile == NULL) {
+			if (expectingUploadData && shouldBufferUploads && uploadDataFile == NULL) {
 				requestData.push_back(uploadDataMemory);
 			}
 			
 			FileDescriptor conn = connectToHelperAgent();
 			gatheredWrite(conn, &requestData[0], requestData.size());
 			
-			if (expectingUploadData && uploadDataFile != NULL) {
-				sendRequestBody(conn, uploadDataFile);
-				uploadDataFile.reset();
+			if (expectingUploadData) {
+				if (shouldBufferUploads && uploadDataFile != NULL) {
+					sendRequestBody(conn, uploadDataFile);
+					uploadDataFile.reset();
+				} else if (!shouldBufferUploads) {
+					sendRequestBody(conn, r);
+				}
 			}
 			
 			do {
@@ -627,7 +645,7 @@ private:
 			/* Setup the bucket brigade. */
 			bb = apr_brigade_create(r->connection->pool, r->connection->bucket_alloc);
 			
-			bucketState = make_shared<PassengerBucketState>(conn);
+			bucketState = boost::make_shared<PassengerBucketState>(conn);
 			b = passenger_bucket_create(bucketState, r->connection->bucket_alloc, config->getBufferResponse());
 			APR_BRIGADE_INSERT_TAIL(bb, b);
 			
@@ -666,83 +684,25 @@ private:
 				}
 				apr_table_setn(r->headers_out, "Status", r->status_line);
 				
-				//bool xsendfile = hasModXsendfile() &&
-				//	apr_table_get(r->err_headers_out, "X-Sendfile");
-				
 				UPDATE_TRACE_POINT();
-				ap_pass_brigade(r->output_filters, bb);
-				
-				/*
-				if (r->connection->aborted) {
-					P_WARN("Either the visitor clicked on the 'Stop' button in the "
-						"web browser, or the visitor's connection has stalled "
-						"and couldn't receive the data that Apache is sending "
-						"to it. As a result, you will probably see a 'Broken Pipe' "
-						"error in this log file. Please ignore it, "
-						"this is normal. You might also want to increase Apache's "
-						"TimeOut configuration option if you experience this "
-						"problem often.");
-				} else if (!bucketState->completed && !xsendfile) {
-					// mod_xsendfile drops the entire output bucket so
-					// suppress this message when mod_xsendfile is active.
-					P_WARN("Apache stopped forwarding the backend's response, "
-						"even though the HTTP client did not close the "
-						"connection. Is this an Apache bug?");
+				if (config->errorOverride == DirConfig::ENABLED
+				 && ap_is_HTTP_ERROR(r->status))
+				{
+					/* Send ErrorDocument.
+					 * Clear r->status for override error, otherwise ErrorDocument
+					 * thinks that this is a recursive error, and doesn't find the
+					 * custom error page.
+					 */
+					int originalStatus = r->status;
+					r->status = HTTP_OK;
+					return originalStatus;
+				} else if (ap_pass_brigade(r->output_filters, bb) == APR_SUCCESS) {
+					apr_brigade_cleanup(bb);
 				}
-				*/
-				
 				return OK;
-			} else if (backendData[0] == '\0') {
-				/* if ((long long) timer.elapsed() >= r->server->timeout / 1000) {
-					// Looks like an I/O timeout.
-					P_ERROR("No data received from " <<
-						"the backend application (process " <<
-						backendPid << ") within " <<
-						(r->server->timeout / 1000) << " msec. Either " <<
-						"the backend application is frozen, or " <<
-						"your TimeOut value of " <<
-						(r->server->timeout / 1000000) <<
-						" seconds is too low. Please check " <<
-						"whether your application is frozen, or " <<
-						"increase the value of the TimeOut " <<
-						"configuration directive.");
-				} else {
-					P_ERROR("The backend application (process " <<
-						backendPid << ") did not send a valid " <<
-						"HTTP response; instead, it sent nothing " <<
-						"at all. It is possible that it has crashed; " <<
-						"please check whether there are crashing " <<
-						"bugs in this application.");
-				} */
-				apr_table_setn(r->err_headers_out, "Status", "500 Internal Server Error");
-				return HTTP_INTERNAL_SERVER_ERROR;
 			} else {
-				/* if ((long long) timer.elapsed() >= r->server->timeout / 1000) {
-					// Looks like an I/O timeout.
-					P_ERROR("The backend application (process " <<
-						backendPid << ") hasn't sent a valid " <<
-						"HTTP response within " <<
-						(r->server->timeout / 1000) << " msec. Either " <<
-						"the backend application froze while " <<
-						"sending a response, or " <<
-						"your TimeOut value of " <<
-						(r->server->timeout / 1000000) <<
-						" seconds is too low. Please check " <<
-						"whether the application is frozen, or " <<
-						"increase the value of the TimeOut " <<
-						"configuration directive. The application " <<
-						"has sent this data so far: [" <<
-						backendData << "]");
-				} else {
-					P_ERROR("The backend application (process " <<
-						backendPid << ") didn't send a valid " <<
-						"HTTP response. It might have crashed " <<
-						"during the middle of sending an HTTP " <<
-						"response, so please check whether there " <<
-						"are crashing problems in your application. " <<
-						"This is the data that it sent: [" <<
-						backendData << "]");
-				} */
+				// HelperAgent sent an empty response, or an invalid response.
+				apr_brigade_cleanup(bb);
 				apr_table_setn(r->err_headers_out, "Status", "500 Internal Server Error");
 				return HTTP_INTERNAL_SERVER_ERROR;
 			}
@@ -870,6 +830,26 @@ private:
 			headers.append(1, '\0');
 		}
 	}
+
+	void addHeader(request_rec *r, string &headers, const char *name, int value) {
+		if (value != UNSET_INT_VALUE) {
+			headers.append(name);
+			headers.append(1, '\0');
+			headers.append(apr_psprintf(r->pool, "%d", value));
+			headers.append(1, '\0');
+		}
+	}
+
+	void addHeader(request_rec *r, string &headers, const char *name, DirConfig::Threeway value) {
+		if (value != DirConfig::UNSET) {
+			headers.append(name);
+			if (value == DirConfig::ENABLED) {
+				headers.append("\0true\0", 6);
+			} else {
+				headers.append("\0false\0", 7);
+			}
+		}
+	}
 	
 	unsigned int constructHeaders(request_rec *r, DirConfig *config,
 		vector<StaticString> &requestData, DirectoryMapper &mapper,
@@ -884,7 +864,7 @@ private:
 		 */
 		size_t uriLen = strlen(r->uri);
 		unsigned int escaped = escapeUri(NULL, (const unsigned char *) r->uri, uriLen);
-		char escapedUri[uriLen + 2 * escaped + 1];
+		char *escapedUri = (char *) apr_palloc(r->pool, uriLen + 2 * escaped + 1);
 		escapeUri((unsigned char *) escapedUri, (const unsigned char *) r->uri, uriLen);
 		escapedUri[uriLen + 2 * escaped] = '\0';
 		
@@ -970,21 +950,14 @@ private:
 		addHeader(output, "PASSENGER_STATUS_LINE", "false");
 		addHeader(output, "PASSENGER_APP_ROOT", mapper.getAppRoot());
 		addHeader(output, "PASSENGER_APP_GROUP_NAME", config->getAppGroupName(mapper.getAppRoot()));
-		addHeader(output, "PASSENGER_RUBY", config->ruby ? config->ruby : serverConfig.defaultRuby);
-		addHeader(output, "PASSENGER_PYTHON", config->python);
-		addHeader(output, "PASSENGER_ENV", config->getEnvironment());
+		#include "SetHeaders.cpp"
 		addHeader(output, "PASSENGER_SPAWN_METHOD", config->getSpawnMethodString());
-		addHeader(output, "PASSENGER_USER", config->getUser());
-		addHeader(output, "PASSENGER_GROUP", config->getGroup());
+		addHeader(r, output, "PASSENGER_MAX_REQUEST_QUEUE_SIZE", config->maxRequestQueueSize);
 		addHeader(output, "PASSENGER_APP_TYPE", mapper.getApplicationTypeName());
-		addHeader(output, "PASSENGER_MIN_INSTANCES",
-			apr_psprintf(r->pool, "%ld", config->getMinInstances()));
 		addHeader(output, "PASSENGER_MAX_PRELOADER_IDLE_TIME",
 			apr_psprintf(r->pool, "%ld", config->maxPreloaderIdleTime));
 		addHeader(output, "PASSENGER_DEBUGGER", "false");
 		addHeader(output, "PASSENGER_SHOW_VERSION_IN_HEADER", "true");
-		addHeader(output, "PASSENGER_MAX_REQUESTS",
-			apr_psprintf(r->pool, "%ld", config->getMaxRequests()));
 		addHeader(output, "PASSENGER_STAT_THROTTLE_RATE",
 			apr_psprintf(r->pool, "%ld", config->getStatThrottleRate()));
 		addHeader(output, "PASSENGER_RESTART_DIR", config->getRestartDir());
@@ -1178,10 +1151,10 @@ private:
 	 * @throws SystemException
 	 * @throws IOException
 	 */
-	shared_ptr<BufferedUpload> receiveRequestBody(request_rec *r) {
+	boost::shared_ptr<BufferedUpload> receiveRequestBody(request_rec *r) {
 		TRACE_POINT();
 		DirConfig *config = getDirConfig(r);
-		shared_ptr<BufferedUpload> tempFile;
+		boost::shared_ptr<BufferedUpload> tempFile;
 		try {
 			ServerInstanceDir::GenerationPtr generation = agentsStarter.getGeneration();
 			string uploadBufferDir = config->getUploadBufferDir(generation);
@@ -1237,7 +1210,7 @@ private:
 		}
 	}
 	
-	void sendRequestBody(const FileDescriptor &fd, shared_ptr<BufferedUpload> &uploadData) {
+	void sendRequestBody(const FileDescriptor &fd, boost::shared_ptr<BufferedUpload> &uploadData) {
 		TRACE_POINT();
 		rewind(uploadData->handle);
 		while (!feof(uploadData->handle)) {
@@ -1245,7 +1218,37 @@ private:
 			size_t size;
 			
 			size = fread(buf, 1, sizeof(buf), uploadData->handle);
-			writeExact(fd, buf, size);
+			try {
+				writeExact(fd, buf, size);
+			} catch (const SystemException &e) {
+				if (e.code() == EPIPE || e.code() == ECONNRESET) {
+					// The HelperAgent stopped reading the body, probably
+					// because the application already sent EOF.
+					return;
+				} else {
+					throw e;
+				}
+			}
+		}
+	}
+
+	void sendRequestBody(const FileDescriptor &fd, request_rec *r) {
+		TRACE_POINT();
+		char buf[1024 * 32];
+		apr_off_t len;
+		
+		try {
+			while ((len = readRequestBodyFromApache(r, buf, sizeof(buf))) > 0) {
+				writeExact(fd, buf, len);
+			}
+		} catch (const SystemException &e) {
+			if (e.code() == EPIPE || e.code() == ECONNRESET) {
+				// The HelperAgent stopped reading the body, probably
+				// because the application already sent EOF.
+				return;
+			} else {
+				throw e;
+			}
 		}
 	}
 
@@ -1288,7 +1291,6 @@ public:
 			.set    ("default_ruby", serverConfig.defaultRuby)
 			.setInt ("max_pool_size", serverConfig.maxPoolSize)
 			.setInt ("pool_idle_time", serverConfig.poolIdleTime)
-			.setInt ("max_instances_per_app", serverConfig.maxInstancesPerApp)
 			.set    ("analytics_log_user", serverConfig.analyticsLogUser)
 			.set    ("analytics_log_group", serverConfig.analyticsLogGroup)
 			.set    ("union_station_gateway_address", serverConfig.unionStationGatewayAddress)
@@ -1577,7 +1579,9 @@ init_module(apr_pool_t *pconf, apr_pool_t *plog, apr_pool_t *ptemp, server_rec *
 	 * universal, i.e. it works for some people but not for others. So we got rid of the
 	 * hacks, and now we always initialize in the post_config hook.
 	 */
-	if (hooks != NULL) {
+	if (hooks == NULL) {
+		oxt::initialize();
+	} else {
 		P_DEBUG("Restarting Phusion Passenger....");
 		delete hooks;
 		hooks = NULL;
@@ -1598,6 +1602,7 @@ init_module(apr_pool_t *pconf, apr_pool_t *plog, apr_pool_t *ptemp, server_rec *
 	} catch (const thread_resource_error &e) {
 		struct rlimit lim;
 		string pthread_threads_max;
+		int ret;
 		
 		lim.rlim_cur = 0;
 		lim.rlim_max = 0;
@@ -1631,11 +1636,13 @@ init_module(apr_pool_t *pconf, apr_pool_t *plog, apr_pool_t *ptemp, server_rec *
 		
 		fprintf(stderr, "Output of 'uname -a' follows:\n");
 		fflush(stderr);
-		::system("uname -a >&2");
+		ret = ::system("uname -a >&2");
+		(void) ret; // Ignore compiler warning.
 		
 		fprintf(stderr, "\nOutput of 'ulimit -a' follows:\n");
 		fflush(stderr);
-		::system("ulimit -a >&2");
+		ret = ::system("ulimit -a >&2");
+		(void) ret; // Ignore compiler warning.
 		
 		return DECLINED;
 		
