@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2011-2013 Phusion
+ *  Copyright (c) 2011-2014 Phusion
  *
  *  "Phusion Passenger" is a trademark of Hongli Lai & Ninh Bui.
  *
@@ -63,7 +63,8 @@ using namespace oxt;
  * so only access within ApplicationPool lock.
  */
 class Group: public boost::enable_shared_from_this<Group> {
-private:
+// Actually private, but marked public so that unit tests can access the fields.
+public:
 	friend class Pool;
 	friend class SuperGroup;
 	
@@ -509,6 +510,7 @@ private:
 		} else if (&destination == &detachedProcesses) {
 			assert(process->isAlive());
 			process->enabled = Process::DETACHED;
+			process->abortLongRunningConnections();
 		} else {
 			P_BUG("Unknown destination list");
 		}
