@@ -35,7 +35,7 @@ using namespace boost;
 using namespace oxt;
 
 
-class SmartSpawner: public Spawner, public enable_shared_from_this<SmartSpawner> {
+class SmartSpawner: public Spawner, public boost::enable_shared_from_this<SmartSpawner> {
 private:
 	/**
 	 * Structure containing arguments and working state for negotiating
@@ -49,16 +49,12 @@ private:
 		BackgroundIOCapturerPtr stderrCapturer;
 		DebugDirPtr debugDir;
 		const Options *options;
-		bool forwardStderr;
-		int forwardStderrTo;
 
 		/****** Working state ******/
 		unsigned long long timeout;
 
 		StartupDetails() {
 			options = NULL;
-			forwardStderr = false;
-			forwardStderrTo = STDERR_FILENO;
 			timeout = 0;
 		}
 	};
@@ -159,7 +155,8 @@ private:
 		// remaining stderr output for at most 2 seconds.
 		if (errorKind != SpawnException::PRELOADER_STARTUP_TIMEOUT
 		 && errorKind != SpawnException::APP_STARTUP_TIMEOUT
-		 && stderrCapturer != NULL) {
+		 && stderrCapturer != NULL)
+		{
 			bool done = false;
 			unsigned long long timeout = 2000;
 			while (!done) {
@@ -186,7 +183,10 @@ private:
 		
 		// Now throw SpawnException with the captured stderr output
 		// as error response.
-		SpawnException e(msg, stderrOutput, false, errorKind);
+		SpawnException e(msg,
+			createErrorPageFromStderrOutput(msg, errorKind, stderrOutput),
+			true,
+			errorKind);
 		e.setPreloaderCommand(getPreloaderCommandString());
 		annotatePreloaderException(e, debugDir);
 		throw e;
@@ -215,7 +215,7 @@ private:
 		vector<string> command = createRealPreloaderCommand(options, args);
 		SocketPair adminSocket = createUnixSocketPair();
 		Pipe errorPipe = createPipe();
-		DebugDirPtr debugDir = make_shared<DebugDir>(preparation.uid, preparation.gid);
+		DebugDirPtr debugDir = boost::make_shared<DebugDir>(preparation.uid, preparation.gid);
 		pid_t pid;
 		
 		pid = syscalls::fork();
@@ -264,13 +264,13 @@ private:
 			details.stderrCapturer =
 				make_shared<BackgroundIOCapturer>(
 					errorPipe.first,
-					string("[App ") + toString(pid) + " stderr] ",
-					config->forwardStderr);
+					pid,
+					// The cast works around a compilation problem in Clang.
+					(const char *) "stderr");
 			details.stderrCapturer->start();
 			details.debugDir = debugDir;
 			details.options = &options;
 			details.timeout = options.startTimeout * 1000;
-			details.forwardStderr = config->forwardStderr;
 			
 			{
 				this_thread::restore_interruption ri(di);
@@ -285,13 +285,13 @@ private:
 			
 			PipeWatcherPtr watcher;
 
-			watcher = make_shared<PipeWatcher>(adminSocket.second,
-				"stdout", pid, config->forwardStdout);
+			watcher = boost::make_shared<PipeWatcher>(adminSocket.second,
+				"stdout", pid);
 			watcher->initialize();
 			watcher->start();
 
-			watcher = make_shared<PipeWatcher>(errorPipe.first,
-				"stderr", pid, config->forwardStderr);
+			watcher = boost::make_shared<PipeWatcher>(errorPipe.first,
+				"stderr", pid);
 			watcher->initialize();
 			watcher->start();
 			
@@ -529,11 +529,19 @@ private:
 	}
 	
 	void handleInvalidResponseType(StartupDetails &details, const string &line) {
-		throwPreloaderSpawnException("An error occurred while starting up "
-			"the preloader. It sent an unknown response type \"" +
-			cEscapeString(line) + "\".",
-			SpawnException::PRELOADER_STARTUP_PROTOCOL_ERROR,
-			details);
+		if (line.empty()) {
+			throwPreloaderSpawnException("An error occurred while starting up "
+				"the preloader. It exited before signalling successful "
+				"startup back to " PROGRAM_NAME ".",
+				SpawnException::PRELOADER_STARTUP_ERROR,
+				details);
+		} else {
+			throwPreloaderSpawnException("An error occurred while starting up "
+				"the preloader. It sent an unknown response type \"" +
+				cEscapeString(line) + "\".",
+				SpawnException::PRELOADER_STARTUP_PROTOCOL_ERROR,
+				details);
+		}
 	}
 	
 	string negotiatePreloaderStartup(StartupDetails &details) {
@@ -715,7 +723,7 @@ public:
 		m_lastUsed = SystemTime::getUsec();
 
 		if (_config == NULL) {
-			config = make_shared<SpawnerConfig>();
+			config = boost::make_shared<SpawnerConfig>();
 		} else {
 			config = _config;
 		}
@@ -765,7 +773,6 @@ public:
 		details.adminSocket = result.adminSocket;
 		details.io = result.io;
 		details.options = &options;
-		details.forwardStderr = config->forwardStderr;
 		ProcessPtr process = negotiateSpawn(details);
 		P_DEBUG("Process spawning done: appRoot=" << options.appRoot <<
 			", pid=" << process->pid);
