@@ -39,9 +39,10 @@
 	#include <mach/mach_vm.h>
 	#include <mach/mach_port.h>
 #endif
-#ifndef __NetBSD__
+#if !defined(__NetBSD__) && !defined(__OpenBSD__)
 	// NetBSD does not support -p with multiple PIDs.
 	// https://code.google.com/p/phusion-passenger/issues/detail?id=736
+	// OpenBSD 5.2 doesn't support it either
 	#define PS_SUPPORTS_MULTIPLE_PIDS
 	#include <set>
 #endif
@@ -92,6 +93,7 @@ struct ProcessMetrics {
 	/** OS X Snow Leopard does not report the VM size correctly, so don't use this. */
 	size_t  vmsize;
 	pid_t   processGroupId;
+	uid_t   uid;
 	string  command;
 	
 	ProcessMetrics() {
@@ -218,10 +220,8 @@ private:
 		*data = endOfWord;
 		return result;
 	}
-	
-	static long long readNextWordAsLongLong(const char **data) {
-		StaticString word = readNextWord(data);
-		char nullTerminatedWord[word.size() + 1];
+
+	static long long processNextWordAsLongLong(const StaticString &word, char *nullTerminatedWord) {
 		memcpy(nullTerminatedWord, word.c_str(), word.size());
 		nullTerminatedWord[word.size()] = '\0';
 		if (*nullTerminatedWord == '\0') {
@@ -231,9 +231,18 @@ private:
 		}
 	}
 	
-	static int readNextWordAsInt(const char **data) {
+	static long long readNextWordAsLongLong(const char **data) {
 		StaticString word = readNextWord(data);
-		char nullTerminatedWord[word.size() + 1];
+		if (word.size() < 50) {
+			char nullTerminatedWord[50];
+			return processNextWordAsLongLong(word, nullTerminatedWord);
+		} else {
+			string nullTerminatedWord(word.size() + 1, '\0');
+			return processNextWordAsLongLong(word, &nullTerminatedWord[0]);
+		}
+	}
+
+	static int processNextWordAsInt(const StaticString &word, char *nullTerminatedWord) {
 		memcpy(nullTerminatedWord, word.c_str(), word.size());
 		nullTerminatedWord[word.size()] = '\0';
 		if (*nullTerminatedWord == '\0') {
@@ -243,6 +252,17 @@ private:
 		}
 	}
 	
+	static int readNextWordAsInt(const char **data) {
+		StaticString word = readNextWord(data);
+		if (word.size() < 50) {
+			char nullTerminatedWord[50];
+			return processNextWordAsInt(word, nullTerminatedWord);
+		} else {
+			string nullTerminatedWord(word.size() + 1, '\0');
+			return processNextWordAsInt(word, &nullTerminatedWord[0]);
+		}
+	}
+	
 	string runCommandAndCaptureOutput(const char **command) const {
 		pid_t pid;
 		int e;
@@ -362,6 +382,7 @@ private:
 			metrics.rss  = (size_t) readNextWordAsLongLong(&start);
 			metrics.vmsize  = (size_t) readNextWordAsLongLong(&start);
 			metrics.processGroupId = (pid_t) readNextWordAsLongLong(&start);
+			metrics.uid  = (uid_t) readNextWordAsLongLong(&start);
 			metrics.command = readRestOfLine(start);
 
 			bool pidAllowed;
@@ -418,25 +439,27 @@ public:
 		}
 		
 		ConstIterator it;
-		string pidsArg;
+		// The list of PIDs must follow -p without a space.
+		// https://groups.google.com/forum/#!topic/phusion-passenger/WKXy61nJBMA
+		string pidsArg = "-p";
 		
 		for (it = pids.begin(); it != pids.end(); it++) {
 			pidsArg.append(toString(*it));
 			pidsArg.append(",");
 		}
-		if (!pidsArg.empty() && pidsArg[pidsArg.size() - 1] == ',') {
+		if (pidsArg[pidsArg.size() - 1] == ',') {
 			pidsArg.resize(pidsArg.size() - 1);
 		}
 		
 		const char *command[] = {
 			"ps", "-o",
 			#if defined(sun) || defined(__sun)
-				"pid,ppid,pcpu,rss,vsz,pgid,args",
+				"pid,ppid,pcpu,rss,vsz,pgid,uid,args",
 			#else
-				"pid,ppid,%cpu,rss,vsize,pgid,command",
+				"pid,ppid,%cpu,rss,vsize,pgid,uid,command",
 			#endif
 			#ifdef PS_SUPPORTS_MULTIPLE_PIDS
-				"-p", pidsArg.c_str(),
+				pidsArg.c_str(),
 			#endif
 			NULL
 		};
