@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2010-2013 Phusion
+ *  Copyright (c) 2010-2014 Phusion
  *
  *  "Phusion Passenger" is a trademark of Hongli Lai & Ninh Bui.
  *
@@ -30,6 +30,7 @@
 #include <cstring>
 #include <cassert>
 #include <cerrno>
+#include <stdlib.h>
 #include <unistd.h>
 #include <limits.h>
 #include <pwd.h>
@@ -372,6 +373,8 @@ private:
 				"privilege to that of user '") + username +
 				"': cannot set user ID", e);
 		}
+
+		setenv("HOME", userEntry->pw_dir, 1);
 	}
 	
 	void onSigquit(ev::sig &signal, int revents) {
@@ -434,7 +437,7 @@ public:
 		accountsDatabase->add("_passenger-status", options.adminToolStatusPassword, false,
 			Account::INSPECT_BASIC_INFO | Account::INSPECT_SENSITIVE_INFO |
 			Account::INSPECT_BACKTRACES | Account::INSPECT_REQUESTS |
-			Account::RESTART);
+			Account::DETACH | Account::RESTART);
 		accountsDatabase->add("_web_server", options.exitPassword, false, Account::EXIT);
 		messageServer = boost::make_shared<MessageServer>(
 			parseUnixSocketAddress(options.adminSocketAddress), accountsDatabase);
@@ -551,7 +554,7 @@ public:
 		poolLoop.start("Pool event loop", 0);
 		requestLoop.start("Request event loop", 0);
 
-		
+
 		/* Wait until the watchdog closes the feedback fd (meaning it
 		 * was killed) or until we receive an exit message.
 		 */
@@ -581,6 +584,9 @@ public:
 			 * inaccessible.
 			 */
 			P_DEBUG("Watchdog seems to be killed; forcing shutdown of all subprocesses");
+			// We send a SIGTERM first to allow processes to gracefully shut down.
+			syscalls::killpg(getpgrp(), SIGTERM);
+			usleep(500000);
 			syscalls::killpg(getpgrp(), SIGKILL);
 			_exit(2); // In case killpg() fails.
 		} else {
@@ -589,6 +595,7 @@ public:
 			 */
 			P_DEBUG("Received command to exit gracefully. "
 				"Waiting until 5 seconds after all clients have disconnected...");
+			pool->prepareForShutdown();
 			requestHandler->resetInactivityTime();
 			while (requestHandler->inactivityTime() < 5000) {
 				syscalls::usleep(250000);
