@@ -696,8 +696,17 @@ private:
 			status, (unsigned long) data.size());
 
 		client->clientOutputPipe->write(header, pos - header);
+		if (!client->connected()) {
+			return;
+		}
 		client->clientOutputPipe->write(data.data(), data.size());
+		if (!client->connected()) {
+			return;
+		}
 		client->clientOutputPipe->end();
+		if (!client->connected()) {
+			return;
+		}
 
 		if (client->useUnionStation()) {
 			snprintf(header, end - header, "Status: %d %s",
@@ -778,8 +787,17 @@ private:
 
 		const string header = str.str();
 		client->clientOutputPipe->write(header.data(), header.size());
+		if (!client->connected()) {
+			return;
+		}
 		client->clientOutputPipe->write(data.data(), data.size());
+		if (!client->connected()) {
+			return;
+		}
 		client->clientOutputPipe->end();
+		if (!client->connected()) {
+			return;
+		}
 
 		if (client->useUnionStation()) {
 			client->logMessage("Status: 500 Internal Server Error");
@@ -1094,6 +1112,10 @@ private:
 	void writeToClientOutputPipe(const ClientPtr &client, const StaticString &data) {
 		bool wasCommittingToDisk = client->clientOutputPipe->isCommittingToDisk();
 		bool nowCommittingToDisk = !client->clientOutputPipe->write(data.data(), data.size());
+		if (!client->connected()) {
+			// EPIPE/ECONNRESET detected.
+			return;
+		}
 		if (!wasCommittingToDisk && nowCommittingToDisk) {
 			RH_TRACE(client, 3, "Buffering response data to disk; temporarily stopping application socket.");
 			client->backgroundOperations++;
@@ -2010,6 +2032,7 @@ private:
 	}
 
 	void sessionCheckedOut_real(ClientPtr client, const SessionPtr &session, const ExceptionPtr &e) {
+		RH_LOG_EVENT(client, "sessionCheckedOut");
 		if (!client->connected()) {
 			return;
 		}
@@ -2200,7 +2223,6 @@ private:
 			data.append(" ");
 			data.append(parser.getHeader("REQUEST_URI"));
 			data.append(" HTTP/1.1\r\n");
-			data.append("Connection: close\r\n");
 
 			for (it = parser.begin(); it != end; it++) {
 				if (startsWith(it->first, "HTTP_") && it->first != "HTTP_CONNECTION") {
@@ -2221,6 +2243,15 @@ private:
 				}
 			}
 
+			StaticString connection = parser.getHeader("HTTP_CONNECTION");
+			if (connection == "upgrade" || connection == "Upgrade") {
+				data.append("Connection: ");
+				data.append(connection.data(), connection.size());
+				data.append("\r\n");
+			} else {
+				data.append("Connection: close\r\n");
+			}
+
 			StaticString header = parser.getHeader("CONTENT_LENGTH");
 			if (!header.empty()) {
 				data.append("Content-Length: ");
