@@ -74,7 +74,7 @@ private:
 	static const int GARBAGE_COLLECTION_TIMEOUT = 4500;  // 1 hour 15 minutes
 	
 	struct LogSink;
-	typedef shared_ptr<LogSink> LogSinkPtr;
+	typedef boost::shared_ptr<LogSink> LogSinkPtr;
 	typedef map<string, LogSinkPtr> LogSinkCache;
 	
 	struct LogSink {
@@ -183,7 +183,7 @@ private:
 		}
 	};
 	
-	typedef shared_ptr<LogFileSink> LogFileSinkPtr;
+	typedef boost::shared_ptr<LogFileSink> LogFileSinkPtr;
 	
 	struct RemoteSink: public LogSink {
 		/* RemoteSender compresses the data with zlib before sending it
@@ -363,7 +363,7 @@ private:
 		}
 	};
 	
-	typedef shared_ptr<Transaction> TransactionPtr;
+	typedef boost::shared_ptr<Transaction> TransactionPtr;
 	
 	enum ClientType {
 		UNINITIALIZED,
@@ -389,12 +389,28 @@ private:
 			type = UNINITIALIZED;
 			dataReader.setMaxSize(1024 * 128);
 		}
+
+		template<typename Stream>
+		void inspect(Stream &stream) const {
+			stream << "   * Client " << (int) fd << "\n";
+			stream << "     Initialized      : " << bool(type == LOGGER) << "\n";
+			stream << "     Node name        : " << nodeName << "\n";
+			stream << "     Open transactions: (" << openTransactions.size() << ")";
+			set<string>::const_iterator it, end = openTransactions.end();
+			for (it = openTransactions.begin(); it != end; it++) {
+				stream << " " << *it;
+			}
+			stream << "\n";
+			stream << "     Connection state : " << getStateName() << "\n";
+			stream << "     Message state    : " << messageServer.getStateName() << "\n";
+			stream << "     Outbox           : " << outbox.size() << " bytes\n";
+		}
 	};
 	
-	typedef shared_ptr<Client> ClientPtr;
+	typedef boost::shared_ptr<Client> ClientPtr;
 	typedef map<string, TransactionPtr> TransactionMap;
 	
-	typedef shared_ptr<FilterSupport::Filter> FilterPtr;
+	typedef boost::shared_ptr<FilterSupport::Filter> FilterPtr;
 	
 	RemoteSender remoteSender;
 	ev::timer garbageCollectionTimer;
@@ -527,7 +543,7 @@ private:
 		LogSinkCache::iterator it = logSinkCache.find(cacheKey);
 		if (it == logSinkCache.end()) {
 			trimLogSinkCache(MAX_LOG_SINK_CACHE_SIZE - 1);
-			result = make_shared<LogFileSink>(this, dumpFile);
+			result = boost::make_shared<LogFileSink>(this, dumpFile);
 			pair<LogSinkCache::iterator, bool> p =
 				logSinkCache.insert(make_pair(cacheKey, result));
 			result->cacheIterator = p.first;
@@ -557,7 +573,7 @@ private:
 		LogSinkCache::iterator it = logSinkCache.find(cacheKey);
 		if (it == logSinkCache.end()) {
 			trimLogSinkCache(MAX_LOG_SINK_CACHE_SIZE - 1);
-			result = make_shared<RemoteSink>(this, unionStationKey,
+			result = boost::make_shared<RemoteSink>(this, unionStationKey,
 				nodeName, category);
 			pair<LogSinkCache::iterator, bool> p =
 				logSinkCache.insert(make_pair(cacheKey, result));
@@ -609,7 +625,7 @@ private:
 		// TODO: garbage collect filters based on time
 		FilterPtr filter = filters.get(source);
 		if (filter == NULL) {
-			filter = make_shared<FilterSupport::Filter>(source);
+			filter = boost::make_shared<FilterSupport::Filter>(source);
 			filters.set(source, filter);
 		}
 		return *filter;
@@ -639,15 +655,6 @@ private:
 		char writeCountStr[sizeof(unsigned int) * 2 + 1];
 		integerToHexatri(transaction->writeCount, writeCountStr);
 		transaction->writeCount++;
-		transaction->data.reserve(transaction->data.size() +
-			transaction->txnId.size() +
-			1 +
-			timestamp.size() +
-			1 +
-			strlen(writeCountStr) +
-			1 +
-			data.size() +
-			1);
 		transaction->data.append(transaction->txnId);
 		transaction->data.append(" ");
 		transaction->data.append(timestamp);
@@ -687,7 +694,7 @@ private:
 	}
 	
 	bool isDirectory(const string &dir, struct dirent *entry) const {
-		#if defined(__sun__) || defined(_AIX)
+		#if defined(__sun__) || defined(sun) || defined(_AIX)
 			string path = dir;
 			path.append("/");
 			path.append(entry->d_name);
@@ -862,8 +869,8 @@ protected:
 					return true;
 				}
 				
-				transaction = make_shared<Transaction>(this, ev_now(getLoop()));
-				if (unionStationKey.empty()) {
+				transaction = boost::make_shared<Transaction>(this, ev_now(getLoop()));
+				if (unionStationKey.empty() || unionStationKey == "-") {
 					char tempNodeId[MD5_HEX_SIZE];
 					
 					if (nodeId == NULL) {
@@ -1156,7 +1163,13 @@ public:
 		TransactionMap::const_iterator it;
 		TransactionMap::const_iterator end = transactions.end();
 		
-		stream << "Number of clients : " << getClients().size() << "\n";
+		ClientSet::const_iterator cit, cend = getClients().end();
+		stream << "Clients:\n";
+		stream << "  Count: " << getClients().size() << "\n";
+		for (cit = getClients().begin(); cit != cend; cit++) {
+			const Client *client = static_cast<Client *>(*cit);
+			client->inspect(stream);
+		}
 		stream << "\n";
 
 		stream << "RemoteSender:\n";
@@ -1183,7 +1196,7 @@ public:
 	}
 };
 
-typedef shared_ptr<LoggingServer> LoggingServerPtr;
+typedef boost::shared_ptr<LoggingServer> LoggingServerPtr;
 
 
 } // namespace Passenger
