@@ -303,6 +303,19 @@ readCleanupPids(const WorkingObjectsPtr &wo) {
 	return result;
 }
 
+static void
+killCleanupPids(const vector<pid_t> &cleanupPids) {
+	foreach (pid_t pid, cleanupPids) {
+		P_DEBUG("Sending SIGTERM to cleanup PID " << pid);
+		kill(pid, SIGTERM);
+	}
+}
+
+static void
+killCleanupPids(const WorkingObjectsPtr &wo) {
+	killCleanupPids(readCleanupPids(wo));
+}
+
 static void
 cleanupAgentsInBackground(const WorkingObjectsPtr &wo, vector<AgentWatcherPtr> &watchers, char *argv[]) {
 	this_thread::disable_interruption di;
@@ -389,10 +402,8 @@ cleanupAgentsInBackground(const WorkingObjectsPtr &wo, vector<AgentWatcherPtr> &
 			wo->serverInstanceDir->destroy();
 
 			// Notify given PIDs about our shutdown.
-			foreach (pid_t pid, cleanupPids) {
-				P_DEBUG("Sending SIGTERM to cleanup PID " << pid);
-				kill(pid, SIGTERM);
-			}
+			killCleanupPids(cleanupPids);
+
 			strcpy(argv[0], "PassengerWatchdog (cleaning up 6...)");
 			_exit(0);
 		} catch (const std::exception &e) {
@@ -418,7 +429,7 @@ cleanupAgentsInBackground(const WorkingObjectsPtr &wo, vector<AgentWatcherPtr> &
 }
 
 static void
-forceAllAgentsShutdown(vector<AgentWatcherPtr> &watchers) {
+forceAllAgentsShutdown(const WorkingObjectsPtr &wo, vector<AgentWatcherPtr> &watchers) {
 	vector<AgentWatcherPtr>::iterator it;
 	
 	for (it = watchers.begin(); it != watchers.end(); it++) {
@@ -428,6 +439,7 @@ forceAllAgentsShutdown(vector<AgentWatcherPtr> &watchers) {
 	for (it = watchers.begin(); it != watchers.end(); it++) {
 		(*it)->forceShutdown();
 	}
+	killCleanupPids(wo);
 }
 
 static string
@@ -640,7 +652,7 @@ startAgents(const WorkingObjectsPtr &wo, vector<AgentWatcherPtr> &watchers) {
 				"Watchdog startup error",
 				e.what(),
 				NULL);
-			forceAllAgentsShutdown(watchers);
+			forceAllAgentsShutdown(wo, watchers);
 			exit(1);
 		}
 		// Allow other exceptions to propagate and crash the watchdog.
@@ -657,7 +669,7 @@ beginWatchingAgents(const WorkingObjectsPtr &wo, vector<AgentWatcherPtr> &watche
 				"Watchdog startup error",
 				e.what(),
 				NULL);
-			forceAllAgentsShutdown(watchers);
+			forceAllAgentsShutdown(wo, watchers);
 			exit(1);
 		}
 		// Allow other exceptions to propagate and crash the watchdog.
@@ -701,6 +713,9 @@ main(int argc, char *argv[]) {
 			"Watchdog startup error",
 			e.what(),
 			NULL);
+		if (wo != NULL) {
+			killCleanupPids(wo);
+		}
 		return 1;
 	}
 	// Allow other exceptions to propagate and crash the watchdog.
@@ -736,7 +751,7 @@ main(int argc, char *argv[]) {
 			cleanupAgentsInBackground(wo, watchers, argv);
 		} else {
 			UPDATE_TRACE_POINT();
-			forceAllAgentsShutdown(watchers);
+			forceAllAgentsShutdown(wo, watchers);
 		}
 		UPDATE_TRACE_POINT();
 		runHookScriptAndThrowOnError("after_watchdog_shutdown");
