@@ -87,6 +87,16 @@ file_exists(const u_char *filename, unsigned int throttle_rate) {
     return get_file_type(filename, throttle_rate) == FT_FILE;
 }
 
+static int
+mapped_filename_equals(const u_char *filename, size_t filename_len, ngx_str_t *str)
+{
+    return (str->len == filename_len &&
+            memcmp(str->data, filename, filename_len) == 0) ||
+           (str->len == filename_len - 1 &&
+            filename[filename_len - 1] == '/' &&
+            memcmp(str->data, filename, filename_len - 1) == 0);
+}
+
 /**
  * Maps the URI for the given request to a page cache file, if possible.
  *
@@ -115,16 +125,12 @@ map_uri_to_page_cache_file(ngx_http_request_t *r, ngx_str_t *public_dir,
     
     /* From this point on we know that filename is not an empty string. */
     
-    /* Check whether filename is equal to public_dir. filename may also be equal to
-     * public_dir + "/" so check for that as well.
+    
+    /* Check whether `filename` is equal to public_dir.
+     * `filename` may also be equal to public_dir + "/" so check for that as well.
      */
-    if ((public_dir->len == filename_len && memcmp(public_dir->data, filename,
-                                                   filename_len) == 0) ||
-        (public_dir->len == filename_len - 1 &&
-         filename[filename_len - 1] == '/' &&
-         memcmp(public_dir->data, filename, filename_len - 1) == 0)
-       ) {
-        /* If the URI maps to the 'public' directory (i.e. the request is the
+    if (mapped_filename_equals(filename, filename_len, public_dir)) {
+        /* If the URI maps to the 'public' or the alias directory (i.e. the request is the
          * base URI) then index.html is the page cache file.
          */
         
@@ -275,63 +281,6 @@ fix_peer_address(ngx_http_request_t *r) {
 }
 
 
-/* Convenience macros for building the SCGI header in create_request(). */
-
-#define ANALYZE_BOOLEAN_CONFIG_LENGTH(name, container, config_field)    \
-    do {                                                                \
-        if (container->config_field) {                                  \
-            len += sizeof(name) + sizeof("true");                       \
-        } else {                                                        \
-            len += sizeof(name) + sizeof("false");                      \
-        }                                                               \
-    } while (0)
-
-#define SERIALIZE_BOOLEAN_CONFIG_DATA(name, container, config_field)    \
-    do {                                                                \
-        b->last = ngx_copy(b->last, name, sizeof(name));                \
-        if (container->config_field) {                                  \
-            b->last = ngx_copy(b->last, "true", sizeof("true"));        \
-        } else {                                                        \
-            b->last = ngx_copy(b->last, "false", sizeof("false"));      \
-        }                                                               \
-    } while (0)
-
-#define PREPARE_INT_CONFIG_DATA(name, container, config_field)      \
-    do {                                                            \
-        end = ngx_snprintf(config_field ## _string,                 \
-                           sizeof(config_field ## _string) - 1,     \
-                           "%d",                                    \
-                           container->config_field);                \
-        *end = '\0';                                                \
-        len += sizeof(name) + (end - config_field ## _string) + 1;  \
-    } while (0)
-
-#define SERIALIZE_INT_CONFIG_DATA(name, container, config_field)        \
-    do {                                                                \
-        b->last = ngx_copy(b->last, name, sizeof(name));                \
-        b->last = ngx_copy(b->last, config_field ## _string,            \
-                           ngx_strlen(config_field ## _string) + 1);    \
-    } while (0)
-
-#define ANALYZE_STR_CONFIG_LENGTH(name, container, config_field)        \
-    do {                                                                \
-        if (container->config_field.data != NULL) {                     \
-            len += sizeof(name) + (container->config_field.len) + 1;    \
-        }                                                               \
-    } while (0)
-
-#define SERIALIZE_STR_CONFIG_DATA(name, container, config_field)        \
-    do {                                                                \
-        if (container->config_field.data != NULL) {                     \
-            b->last = ngx_copy(b->last, name, sizeof(name));            \
-            b->last = ngx_copy(b->last, container->config_field.data,   \
-                               container->config_field.len);            \
-            *b->last = '\0';                                            \
-            b->last++;                                                  \
-        }                                                               \
-    } while (0)
-
-
 #if (NGX_HTTP_CACHE)
 
 static ngx_int_t
@@ -370,10 +319,6 @@ create_request(ngx_http_request_t *r)
     int                            server_name_len;
     ngx_str_t                      escaped_uri;
     ngx_str_t                     *union_station_filters = NULL;
-    u_char                         min_instances_string[12];
-    u_char                         max_requests_string[12];
-    u_char                         max_preloader_idle_time_string[12];
-    u_char                        *end;
     void                          *tmp;
     ngx_uint_t                     i, n;
     ngx_buf_t                     *b;
@@ -386,9 +331,6 @@ create_request(ngx_http_request_t *r)
     passenger_loc_conf_t          *slcf;
     passenger_context_t           *context;
     ngx_http_script_len_code_pt    lcode;
-    #if (NGX_HTTP_SSL)
-        ngx_http_ssl_srv_conf_t   *ssl_conf;
-    #endif
     
     cscf = ngx_http_get_module_srv_conf(r, ngx_http_core_module);
     slcf = ngx_http_get_module_loc_conf(r, ngx_http_passenger_module);
@@ -464,62 +406,16 @@ create_request(ngx_http_request_t *r)
     }
     
     #if (NGX_HTTP_SSL)
-        ssl_conf = ngx_http_get_module_srv_conf(r, ngx_http_ssl_module);
-        if (ssl_conf->enable) {
+        if (r->http_connection->ssl) {
             len += sizeof("HTTPS") + sizeof("on");
         }
     #endif
     
     /* Lengths of Passenger application pool options. */
-    ANALYZE_BOOLEAN_CONFIG_LENGTH("PASSENGER_FRIENDLY_ERROR_PAGES",
-                                  slcf, friendly_error_pages);
-    ANALYZE_BOOLEAN_CONFIG_LENGTH("UNION_STATION_SUPPORT",
-                                  slcf, union_station_support);
-    ANALYZE_BOOLEAN_CONFIG_LENGTH("PASSENGER_DEBUGGER",
-                                  slcf, debugger);
-    ANALYZE_BOOLEAN_CONFIG_LENGTH("PASSENGER_SHOW_VERSION_IN_HEADER",
-                                  slcf, show_version_in_header);
-    if (slcf->ruby.data != NULL) {
-        ANALYZE_STR_CONFIG_LENGTH("PASSENGER_RUBY", slcf, ruby);
-    } else {
-        len += sizeof("PASSENGER_RUBY") + passenger_main_conf.default_ruby.len + 1;
-    }
-    ANALYZE_STR_CONFIG_LENGTH("PASSENGER_PYTHON", slcf, python);
-    len += sizeof("PASSENGER_ENV") + slcf->environment.len + 1;
-    len += sizeof("PASSENGER_SPAWN_METHOD") + slcf->spawn_method.len + 1;
+    len += slcf->options_cache.len;
+
     len += sizeof("PASSENGER_APP_TYPE") + app_type_string_len;
-    ANALYZE_STR_CONFIG_LENGTH("PASSENGER_APP_GROUP_NAME", slcf, app_group_name);
-    ANALYZE_STR_CONFIG_LENGTH("PASSENGER_APP_ROOT", slcf, app_root);
-    ANALYZE_STR_CONFIG_LENGTH("PASSENGER_APP_RIGHTS", slcf, app_rights);
-    ANALYZE_STR_CONFIG_LENGTH("PASSENGER_USER", slcf, user);
-    ANALYZE_STR_CONFIG_LENGTH("PASSENGER_GROUP", slcf, group);
-    ANALYZE_STR_CONFIG_LENGTH("UNION_STATION_KEY", slcf, union_station_key);
-    
-    end = ngx_snprintf(min_instances_string,
-                       sizeof(min_instances_string) - 1,
-                       "%d",
-                       (slcf->min_instances == (ngx_int_t) -1) ? 1 : slcf->min_instances);
-    *end = '\0';
-    len += sizeof("PASSENGER_MIN_INSTANCES") +
-           ngx_strlen(min_instances_string) + 1;
 
-    end = ngx_snprintf(max_requests_string,
-                       sizeof(max_requests_string) - 1,
-                       "%d",
-                       (slcf->max_requests == (ngx_int_t) -1) ? 0 : slcf->max_requests);
-    *end = '\0';
-    len += sizeof("PASSENGER_MAX_REQUESTS") +
-           ngx_strlen(max_requests_string) + 1;
-    
-    end = ngx_snprintf(max_preloader_idle_time_string,
-                       sizeof(max_preloader_idle_time_string) - 1,
-                       "%d",
-                       (slcf->max_preloader_idle_time == (ngx_int_t) -1) ?
-                           -1 : slcf->max_preloader_idle_time);
-    *end = '\0';
-    len += sizeof("PASSENGER_MAX_PRELOADER_IDLE_TIME") +
-           ngx_strlen(max_preloader_idle_time_string) + 1;
-    
     if (slcf->union_station_filters != NGX_CONF_UNSET_PTR && slcf->union_station_filters->nelts > 0) {
         len += sizeof("UNION_STATION_FILTERS");
         
@@ -681,8 +577,7 @@ create_request(ngx_http_request_t *r)
     }
     
     #if (NGX_HTTP_SSL)
-        ssl_conf = ngx_http_get_module_srv_conf(r, ngx_http_ssl_module);
-        if (ssl_conf->enable) {
+        if (r->http_connection->ssl) {
             b->last = ngx_copy(b->last, "HTTPS", sizeof("HTTPS"));
             b->last = ngx_copy(b->last, "on", sizeof("on"));
         }
@@ -690,69 +585,12 @@ create_request(ngx_http_request_t *r)
     
 
     /* Build Passenger application pool option headers. */
-    SERIALIZE_BOOLEAN_CONFIG_DATA("PASSENGER_FRIENDLY_ERROR_PAGES",
-                                  slcf, friendly_error_pages);
-    SERIALIZE_BOOLEAN_CONFIG_DATA("UNION_STATION_SUPPORT",
-                                  slcf, union_station_support);
-    SERIALIZE_BOOLEAN_CONFIG_DATA("PASSENGER_DEBUGGER",
-                                  slcf, debugger);
-    SERIALIZE_BOOLEAN_CONFIG_DATA("PASSENGER_SHOW_VERSION_IN_HEADER",
-                                  slcf, show_version_in_header);
-    
-    if (slcf->ruby.data != NULL) {
-        SERIALIZE_STR_CONFIG_DATA("PASSENGER_RUBY",
-                                  slcf, ruby);
-    } else {
-        b->last = ngx_copy(b->last, "PASSENGER_RUBY",
-                           sizeof("PASSENGER_RUBY"));
-        b->last = ngx_copy(b->last, passenger_main_conf.default_ruby.data,
-                           passenger_main_conf.default_ruby.len + 1);
-    }
-    SERIALIZE_STR_CONFIG_DATA("PASSENGER_PYTHON",
-                              slcf, python);
-
-    b->last = ngx_copy(b->last, "PASSENGER_ENV",
-                       sizeof("PASSENGER_ENV"));
-    b->last = ngx_copy(b->last, slcf->environment.data,
-                       slcf->environment.len + 1);
-
-    b->last = ngx_copy(b->last, "PASSENGER_SPAWN_METHOD",
-                       sizeof("PASSENGER_SPAWN_METHOD"));
-    b->last = ngx_copy(b->last, slcf->spawn_method.data,
-                       slcf->spawn_method.len + 1);
-
-    SERIALIZE_STR_CONFIG_DATA("PASSENGER_APP_GROUP_NAME",
-                              slcf, app_group_name);
-    SERIALIZE_STR_CONFIG_DATA("PASSENGER_APP_ROOT",
-                              slcf, app_root);
-    SERIALIZE_STR_CONFIG_DATA("PASSENGER_APP_RIGHTS",
-                              slcf, app_rights);
-    SERIALIZE_STR_CONFIG_DATA("PASSENGER_USER",
-                              slcf, user);
-    SERIALIZE_STR_CONFIG_DATA("PASSENGER_GROUP",
-                              slcf, group);
-    SERIALIZE_STR_CONFIG_DATA("UNION_STATION_KEY",
-                              slcf, union_station_key);
+    b->last = ngx_copy(b->last, slcf->options_cache.data, slcf->options_cache.len);
 
     b->last = ngx_copy(b->last, "PASSENGER_APP_TYPE",
                        sizeof("PASSENGER_APP_TYPE"));
     b->last = ngx_copy(b->last, app_type_string, app_type_string_len);
 
-    b->last = ngx_copy(b->last, "PASSENGER_MIN_INSTANCES",
-                       sizeof("PASSENGER_MIN_INSTANCES"));
-    b->last = ngx_copy(b->last, min_instances_string,
-                       ngx_strlen(min_instances_string) + 1);
-
-    b->last = ngx_copy(b->last, "PASSENGER_MAX_REQUESTS",
-                       sizeof("PASSENGER_MAX_REQUESTS"));
-    b->last = ngx_copy(b->last, max_requests_string,
-                       ngx_strlen(max_requests_string) + 1);
-
-    b->last = ngx_copy(b->last, "PASSENGER_MAX_PRELOADER_IDLE_TIME",
-                       sizeof("PASSENGER_MAX_PRELOADER_IDLE_TIME"));
-    b->last = ngx_copy(b->last, max_preloader_idle_time_string,
-                       ngx_strlen(max_preloader_idle_time_string) + 1);
-
     if (slcf->union_station_filters != NGX_CONF_UNSET_PTR && slcf->union_station_filters->nelts > 0) {
         b->last = ngx_copy(b->last, "UNION_STATION_FILTERS",
                            sizeof("UNION_STATION_FILTERS"));
@@ -1384,10 +1222,11 @@ passenger_content_handler(ngx_http_request_t *r)
     u_char                *path_last, *end;
     u_char                 root_path_str[NGX_MAX_PATH + 1];
     ngx_str_t              root_path;
-    size_t                 root, len;
+    size_t                 root_len, len;
     u_char                 page_cache_file_str[NGX_MAX_PATH + 1];
     ngx_str_t              page_cache_file;
     passenger_context_t   *context;
+    PP_Error               error;
 
     if (passenger_main_conf.root_dir.len == 0) {
         return NGX_DECLINED;
@@ -1410,7 +1249,7 @@ passenger_content_handler(ngx_http_request_t *r)
     /* Let the next content handler take care of this request if this URL
      * maps to an existing file.
      */
-    path_last = ngx_http_map_uri_to_path(r, &path, &root, 0);
+    path_last = ngx_http_map_uri_to_path(r, &path, &root_len, 0);
     if (path_last != NULL && file_exists(path.data, 0)) {
         return NGX_DECLINED;
     }
@@ -1418,10 +1257,10 @@ passenger_content_handler(ngx_http_request_t *r)
     /* Create a string containing the root path. This path already
      * contains a trailing slash.
      */
-    end = ngx_copy(root_path_str, path.data, root);
+    end = ngx_copy(root_path_str, path.data, root_len);
     *end = '\0';
     root_path.data = root_path_str;
-    root_path.len  = root;
+    root_path.len  = root_len;
     
     
     context = ngx_pcalloc(r->pool, sizeof(passenger_context_t));
@@ -1433,13 +1272,20 @@ passenger_content_handler(ngx_http_request_t *r)
     
     /* Find the base URI for this web application, if any. */
     if (find_base_uri(r, slcf, &base_uri)) {
-        /* Store the found base URI into context->public_dir. We infer that the 'public'
-         * directory of the web application is document root + base URI.
+        /* Store the found base URI into context->public_dir. We infer that
+         * the 'public' directory of the web app equals document root + base URI.
          */
-        len = root_path.len + base_uri.len + 1;
-        context->public_dir.data = ngx_palloc(r->pool, sizeof(u_char) * len);
-        end = ngx_copy(context->public_dir.data, root_path.data, root_path.len);
-        end = ngx_copy(end, base_uri.data, base_uri.len);
+        if (slcf->document_root.data != NULL) {
+            len = slcf->document_root.len + 1;
+            context->public_dir.data = ngx_palloc(r->pool, sizeof(u_char) * len);
+            end = ngx_copy(context->public_dir.data, slcf->document_root.data,
+                           slcf->document_root.len);
+        } else {
+            len = root_path.len + base_uri.len + 1;
+            context->public_dir.data = ngx_palloc(r->pool, sizeof(u_char) * len);
+            end = ngx_copy(context->public_dir.data, root_path.data, root_path.len);
+            end = ngx_copy(end, base_uri.data, base_uri.len);
+        }
         *end = '\0';
         context->public_dir.len = len - 1;
         context->base_uri = base_uri;
@@ -1466,18 +1312,48 @@ passenger_content_handler(ngx_http_request_t *r)
         return passenger_static_content_handler(r, &page_cache_file);
     }
     
-    if (slcf->app_root.data == NULL) {
-        context->app_type = pp_app_type_detector_check_document_root(
-            pp_app_type_detector,
-            (const char *) context->public_dir.data, context->public_dir.len,
-            context->base_uri.len != 0);
+    if (slcf->app_type.data == NULL) {
+        pp_error_init(&error);
+        if (slcf->app_root.data == NULL) {
+            context->app_type = pp_app_type_detector_check_document_root(
+                pp_app_type_detector,
+                (const char *) context->public_dir.data, context->public_dir.len,
+                context->base_uri.len != 0,
+                &error);
+        } else {
+            context->app_type = pp_app_type_detector_check_app_root(
+                pp_app_type_detector,
+                (const char *) slcf->app_root.data, slcf->app_root.len,
+                &error);
+        }
+        if (context->app_type == PAT_NONE) {
+            return NGX_DECLINED;
+        } else if (context->app_type == PAT_ERROR) {
+            if (error.errnoCode == EACCES) {
+                ngx_log_error(NGX_LOG_ALERT, r->connection->log, 0,
+                              "%s; This error means that the Nginx worker process (PID %d, "
+                              "running as UID %d) does not have permission to access this file. "
+                              "Please read the manual to learn how to fix this problem: "
+                              "section 'Troubleshooting' -> 'Upon accessing the web app, Nginx "
+                              "reports a \"Permission denied\" error'; Extra info",
+                              error.message,
+                              (int) getpid(),
+                              (int) getuid());
+            } else {
+                ngx_log_error(NGX_LOG_ALERT, r->connection->log,
+                              (error.errnoCode == PP_NO_ERRNO) ? 0 : error.errnoCode,
+                              "%s",
+                              error.message);
+            }
+            pp_error_destroy(&error);
+            return NGX_HTTP_INTERNAL_SERVER_ERROR;
+        }
     } else {
-        context->app_type = pp_app_type_detector_check_app_root(
-            pp_app_type_detector,
-            (const char *) slcf->app_root.data, slcf->app_root.len);
-    }
-    if (context->app_type == PAT_NONE) {
-        return NGX_DECLINED;
+        context->app_type = pp_get_app_type2((const char *) slcf->app_type.data,
+            slcf->app_type.len);
+        if (context->app_type == PAT_NONE) {
+            return NGX_DECLINED;
+        }
     }
     
     
