@@ -21,7 +21,7 @@
 #  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 #  THE SOFTWARE.
 
-require 'phusion_passenger/platform_info'
+PhusionPassenger.require_passenger_lib 'platform_info'
 
 module PhusionPassenger
 
