@@ -422,11 +422,7 @@ private
 	end
 
 	def should_wait_until_nginx_has_exited?
-		return !@options[:daemonize]
-	end
-
-	def should_cleanup_temp_dir?
-		return @temp_dir && !@options[:daemonize]
+		return !@options[:daemonize] || @app_finder.multi_mode?
 	end
 
 	# Returns the URL that Nginx will be listening on.
